package jmx.service.jndi;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.Name;
import javax.naming.NamingException;
import javax.naming.NameNotFoundException;
import org.jboss.naming.NonSerializableFactory;

/**
 * Abstract MBean implementation to add an object in the JNDI namespace.
 * Used as a super-class for more specific MBean.
 */
public abstract class JNDI implements JNDIMBean
{
    private String  _jndiName;
    private Object  _object;
    private boolean _started;
    private boolean _serializable;

    public JNDI ()
    {
        _started      = false;
        _serializable = true;
        _object       = null;
        _jndiName     = null;
    }

    public String getJndiName ()
    {
        return _jndiName;
    }

    public void setJndiName (String jndiName) throws NamingException
    {
        String oldName = _jndiName;
        _jndiName = jndiName;
        rebind (oldName, _jndiName);
    }
    
    public boolean getIsSerializable ()
    {
        return _serializable;
    }

    public void setIsSerializable (boolean serializable)
    {
        _serializable = serializable;
    }

    public void start() throws Exception
    {
        _object = createObject ();
        bind (_jndiName);
        _started = true;
    }
    
    public void stop ()
    {
        unbind (_jndiName);
        _started = false;
    }

    protected void rebind (String oldName, String newName) throws NamingException
    {
        if (_started) {
            unbind (oldName);
            try {
                bind (newName); 
            }
            catch (Exception e) {
                NamingException ne = new NamingException ("Failed to update jndiName");
                ne.setRootCause (e);
                throw ne;
            }
        }
    }

    protected void bind (String jndiName) throws NamingException
    {
        InitialContext rootCtx = new InitialContext ();
        Name fullName = rootCtx.getNameParser ("").parse (jndiName);
        if (_serializable) {
            createSubcontext (rootCtx, fullName.getPrefix (fullName.size () - 1));
            rootCtx.bind (fullName, _object);
        }
        else {
            NonSerializableFactory.rebind (fullName, _object, true);
        }
    }

    protected void unbind (String jndiName)
    {
        try {
            InitialContext rootCtx = new InitialContext ();
            if (_serializable) {
                rootCtx.unbind (jndiName);
            }
            else {
                NonSerializableFactory.unbind (jndiName);
            }
        }
        catch (NamingException e) {
            // Ignore. Should be an info or warning trace.
            //e.printStackTrace ();
        }
    }

    private void createSubcontext(Context ctx, Name name)
        throws NamingException
    {
        Context subctx = ctx;
        for(int pos = 0; pos < name.size(); pos ++)
        {
            String ctxName = name.get(pos);
            try
            {
                subctx = (Context) ctx.lookup(ctxName);
            }
            catch(NameNotFoundException e)
            {
                subctx = ctx.createSubcontext(ctxName);
            }
            // The current subctx will be the ctx for the next name component
            ctx = subctx;
        }
    }
}
